package graphs;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.IOException;
import java.util.*;

public class Textbook {

    public static Graph searchTextbook(List<String> seeds) throws IOException {
        Stack<String> frontier = new Stack<String>();
        Set<String> visited = new HashSet<>();
        Graph g = new Graph();
        for (String s : seeds) {
            frontier.push(s);

            while (!frontier.isEmpty()) {
                String BASE_URL = "https://www.cis.upenn.edu/~cit5940/";
                String currentLink = frontier.pop();
                if (visited.contains(currentLink)) {
                    continue;
                }
                visited.add(currentLink);
                Vertex currentVertex = g.getVertex(currentLink);
                if (currentVertex == null) {
                    currentVertex = g.addVertex(currentLink);
                }
                Document site = Jsoup.connect(BASE_URL + currentLink).get();
                Elements links = site.getElementsByTag("a");
                for (Element link : links) {
                    // destination of edge
                    String destinationLink = link.attr("href");
                    if (!visited.contains(destinationLink)) {
                        frontier.push(destinationLink);
                    }
                    Vertex destinationVertex = g.getVertex(destinationLink);
                    if (destinationVertex == null) {
                        destinationVertex = g.addVertex(destinationLink);
                    }
                    g.addDirectedEdge(currentVertex, destinationVertex, 1);
                }
            }
        }
        return g;
    }

    public static void topoSort(Graph g) {
        Queue<Vertex> queue = new ArrayDeque<>();
        HashMap<Vertex, Integer> inCounts = new HashMap<>();
        g.getVertices().forEach((v) -> inCounts.put(v, g.getEdgesTo(v).size()));

        for (Map.Entry<Vertex, Integer> e : inCounts.entrySet()) {
            if (e.getValue() == 0) {
                queue.add(e.getKey());
            }
        }

        while (!queue.isEmpty()) {
            Vertex next = queue.poll();
            System.out.println(next);
            for (Edge e : g.getEdgesFrom(next)) {
                inCounts.put(e.toVertex, inCounts.get(e.toVertex) - 1);
                if (inCounts.get(e.toVertex) == 0) {
                    queue.add(e.toVertex);
                }
            }
        }

    }

    public static void main(String[] args) throws IOException {
        Graph textbook = searchTextbook(List.of("chapterA.html", "chapterE.html"));
        System.out.println(textbook.getVertices());
        System.out.println(textbook.getEdges());

        topoSort(textbook);
    }
}
